unit UnForMun ;

interface

uses
  UnConTyp  ; { for type TVect3 TFundArg const SecRad }

procedure MoonEclPos ( t  : Extended ;  var  v , b , d  : Extended ) ;

procedure MoonEquPos ( t  : Extended ;  Var  r  : TVect3 ) ;

procedure MoonFixPos ( t  : Extended ;  Var  r  : TVect3 ) ;

procedure MoonTruPos ( t  : Extended ;  Var  r  : TVect3 ) ;

procedure BrmEquPos ( t , v , b , d  : Extended ;  Var  r  : TVect3 ) ;

procedure BrmFixPos ( t , v , b , d  : Extended ;  Var  r  : TVect3 ) ;

procedure BrmTruPos ( t , v , b , d  : Extended ;  Var  r  : TVect3 ) ;

implementation

uses
  UnForNut , { for procedure ClcFundArg function ToGetEpsMean }
  UnForPnm ; { for procedure ClcPrecMatr Clv}

procedure CorFundArg ( t  : Extended ;  { in modified julian day }
                       f  : TFundArg ;
                       Var  fa  : TFundArg ) ;
var { to add additive terms to fundamental arguments }
  dt  : Extended ;
  s1,s2,s3,s4,s5,s6,s7  : Extended ;
  dr  : Extended ;
begin
  dt:=(t-51544.5)/36525.0;      { interal in julian century }
  s1:=Sin(1.24614+0.35255*dt);  { sinus argument radian     20.1996 degree }
  s2:=Sin(1.75106+0.28325*dt);  { sinus argument radian     16.2288 degree }
  s3:=Sin(1.05727-2.31868*dt);  { sinus argument radian   -132.8508 degree }
  s4:=Sin(2.18240-33.7571*dt);  { sinus argument radian  -1934.1396 degree }
  s5:=Sin(0.65961-33.79719*dt); { sinus argument radian  -1936.4364 degree }
  s6:=Sin(2.68173-2.62983*dt);  { sinus argument radian   -150.678  degree }
  s7:=Sin(0.93890-33.77281*dt); { sinus argument radian  -1935.0396 degree }
  dr:=0.31*s2+14.27*s3;
  fa[1]:=f[1]+SecRad*(0.84*s1+dr+7.26*s4+0.28*s5+0.24*s6); { lambda rd }
  fa[2]:=f[2]+SecRad*(2.94*s1+dr+9.34*s4+1.12*s5+0.83*s6); { l  radian }
  fa[3]:=f[3]-SecRad*(6.4*s1+1.89*s6);                     { l' radian }
  fa[4]:=f[4]+SecRad*(0.21*s1+dr-88.7*s4-15.3*s5+0.24*s6-1.86*s7); { F }
  fa[5]:=f[5]+SecRad*(7.24*s1+dr+7.26*s4+0.28*s5+2.13*s6); { D  radian }
end;

{ approximately series for the Moon }

procedure MoonEclPos ( t  : Extended ;  var  v , b , d  : Extended ) ;
var
  ar,al,ab  : Extended ;
  fa,funda  : TFundArg ; { type from UnConTyp fundamental arguments }
begin
  ClcFundArg(t,funda); { from UnForNut for fundamental arguments radian }
  CorFundArg(t,funda,fa);
  { for range in arcsec }
  ar:= {3422.70}
         +0.260968*Cos(4*fa[5])
        +28.233869*Cos(2*fa[5])
         +0.043566*Cos(fa[2]+4*fa[5])
         +3.08589*Cos(fa[2]+2*fa[5])
       +186.539296*Cos(fa[2])
        +34.311569*Cos(fa[2]-2*fa[5])
         +0.60071*Cos(fa[2]-4*fa[5])
         -0.300334*Cos(fa[3]+2*fa[5])
         -0.399822*Cos(fa[3])
         +1.916735*Cos(fa[3]-2*fa[5])
         +0.034671*Cos(fa[3]-4*fa[5])
         -0.977818*Cos(fa[5])
         +0.282799*Cos(2*fa[2]+2*fa[5])
        +10.165933*Cos(2*fa[2])
         -0.304041*Cos(2*fa[2]-2*fa[5])
         +0.372337*Cos(2*fa[2]-4*fa[5])
         -0.949147*Cos(fa[2]+fa[3])
         +1.443617*Cos(fa[2]+fa[3]-2*fa[5])
         +0.067283*Cos(fa[2]+fa[3]-4*fa[5])
         +0.229935*Cos(fa[2]-fa[3]+2*fa[5])
         +1.152852*Cos(fa[2]-fa[3])
         -0.225821*Cos(fa[2]-fa[3]-2*fa[5])
         -0.008639*Cos(2*fa[3])
         +0.091646*Cos(2*fa[3]-2*fa[5])
         -0.012103*Cos(2*fa[4])
         -0.105291*Cos(2*fa[4]-2*fa[5])
         -0.109456*Cos(fa[2]+fa[5])
         +0.011715*Cos(fa[2]-fa[5])
         -0.038258*Cos(fa[2]-3*fa[5])
         +0.149444*Cos(fa[3]+fa[5])
         -0.225821*Cos(fa[2]-fa[3]-2*fa[5])
         -0.008639*Cos(2*fa[3])
         +0.091646*Cos(2*fa[3]-2*fa[5])
         -0.012103*Cos(2*fa[4])
         -0.105291*Cos(2*fa[4]-2*fa[5])
         -0.109456*Cos(fa[2]+fa[5])
         +0.011715*Cos(fa[2]-fa[5])
         -0.038258*Cos(fa[2]-3*fa[5])
         -0.118714*Cos(3*fa[2]-2*fa[5])
         -0.047853*Cos(fa[2]-2*fa[4]+2*fa[5])
         -0.708093*Cos(fa[2]-2*fa[4])
         -0.048117*Cos(fa[2]+fa[3]+2*fa[5])
         +0.621546*Cos(3*fa[2])
         -0.103337*Cos(2*fa[2]+fa[3])
         -0.083228*Cos(fa[2]+2*fa[4]-2*fa[5]);
  { for longitude in arcsec }
  al:=+13.90*Sin(4*fa[5])
      +2369.92*Sin(2*fa[5])
         +1.98*Sin(fa[2]+4*fa[5])
       +191.96*Sin(fa[2]+2*fa[5])
      -4586.47*Sin(fa[2]-2*fa[5])
        -38.43*Sin(fa[2]-4*fa[5])
        -24.42*Sin(fa[3]+2*fa[5])
       -668.15*Sin(fa[3])
       -165.15*Sin(fa[3]-2*fa[5])
         -1.88*Sin(fa[3]-4*fa[5])
       -125.15*Sin(fa[5])
        +14.38*Sin(2*fa[2]+2*fa[5])
       +769.02*Sin(2*fa[2])
       -211.66*Sin(2*fa[2]-2*fa[5])
        -30.77*Sin(2*fa[2]-4*fa[5])
         -2.92*Sin(fa[2]+fa[3]+2*fa[5])
       -109.67*Sin(fa[2]+fa[3])
       -205.96*Sin(fa[2]+fa[3]-2*fa[5])
         -4.39*Sin(fa[2]+fa[3]-4*fa[5])
        +14.57*Sin(fa[2]-fa[3]+2*fa[5])
       +147.69*Sin(fa[2]-fa[3])
        +28.47*Sin(fa[2]-fa[3]-2*fa[5])
         -7.49*Sin(2*fa[3])
         -8.09*Sin(2*fa[3]-2*fa[5])
         -5.74*Sin(2*fa[4]+2*fa[5])
       -411.60*Sin(2*fa[4])
        -55.17*Sin(2*fa[4]-2*fa[5])
         -8.46*Sin(fa[2]+fa[5])
        +18.61*Sin(fa[2]-fa[5])
         +3.21*Sin(fa[2]-3*fa[5])
        +18.02*Sin(fa[3]+fa[5])
         +0.56*Sin(fa[3]-fa[5])
        +36.12*Sin(3*fa[2])
        -13.19*Sin(3*fa[2]-2*fa[5])
         -7.65*Sin(2*fa[2]+fa[3])
         +9.70*Sin(2*fa[2]-fa[3])
         -2.49*Sin(2*fa[2]-fa[3]-2*fa[5])
         -0.99*Sin(fa[2]+2*fa[4]+2*fa[5])
        -45.10*Sin(fa[2]+2*fa[4])
         -6.38*Sin(fa[2]-2*fa[4]+2*fa[5])
        +39.53*Sin(fa[2]-2*fa[4])
         +1.75*Sin(2*fa[2]-fa[5])
     +22639.50*Sin(fa[2])
         -0.57*Sin(2*fa[2]-6*fa[5])
         +0.64*Sin(fa[2]-1*fa[3]-4*fa[5])
         +1.06*Sin(3*fa[2]+2*fa[5])
         -1.19*Sin(3*fa[2]-4*fa[5])
         -8.63*Sin(2*fa[2]+fa[3]-2*fa[5])
         -2.74*Sin(2*fa[2]+fa[3]-4*fa[5])
         +1.18*Sin(2*fa[2]-fa[3]+2*fa[5])
         -1.17*Sin(fa[2]+2*fa[3])
         -7.41*Sin(fa[2]+2*fa[3]-2*fa[5])
         +0.76*Sin(fa[2]-2*fa[3]+2*fa[5])
         +2.58*Sin(fa[2]-2*fa[3])
         +2.53*Sin(fa[2]-2*fa[3]-2*fa[5])
         +9.37*Sin(fa[2]-2*fa[4]-2*fa[5])
         -2.15*Sin(fa[3]+2*fa[4]-2*fa[5])
         -1.44*Sin(fa[3]-2*fa[4]+2*fa[5])
         -0.59*Sin(2*fa[2]+fa[5])
         +1.22*Sin(2*fa[2]-3*fa[5])
         +1.27*Sin(fa[2]+fa[3]+1*fa[5])
         -1.09*Sin(fa[2]-fa[3]-1*fa[5])
         +0.58*Sin(2*fa[4]-fa[5])
         +1.94*Sin(4*fa[2])
         -0.95*Sin(4*fa[2]-2*fa[5])
         -0.55*Sin(3*fa[2]+fa[3])
         +0.67*Sin(3*fa[2]-fa[3])
         -4.00*Sin(2*fa[2]+2*fa[4])
         +0.56*Sin(2*fa[2]+2*fa[4]-2*fa[5])
         -1.30*Sin(2*fa[2]-2*fa[4])
         +0.54*Sin(2*fa[2]-2*fa[4]-2*fa[5]);
  { for latitude in arcsec }
  ab:=  117.26*Sin(fa[4]+2*fa[5])
     +18461.35*Sin(fa[4])
       -623.66*Sin(fa[4]-2*fa[5])
         -3.67*Sin(fa[4]-4*fa[5])
        +15.12*Sin(fa[2]+fa[4]+2*fa[5])
       -166.58*Sin(fa[2]+fa[4]-2*fa[5])
         -6.58*Sin(fa[2]+fa[4]-4*fa[5])
         +3.00*Sin(-fa[2]+fa[4]+4*fa[5])
       +199.49*Sin(-fa[2]+fa[4]+2*fa[5])
       -999.69*Sin(-fa[2]+fa[4])
        -33.36*Sin(-fa[2]+fa[4]-2*fa[5])
         -6.48*Sin(fa[3]+fa[4])
        -29.65*Sin(fa[3]+fa[4]-2*fa[5])
         +7.98*Sin(-fa[3]+fa[4]+2*fa[5])
         +4.86*Sin(-fa[3]+fa[4])
         -5.38*Sin(fa[4]+fa[5])
         +4.81*Sin(fa[4]-fa[5])
        -15.57*Sin(2*fa[2]+fa[4]-2*fa[5])
        -31.76*Sin(-2*fa[2]+fa[4])
         -5.33*Sin(fa[2]+fa[3]+fa[4])
         +8.89*Sin(-fa[2]-fa[3]+fa[4]+2*fa[5])
         +6.75*Sin(fa[2]-fa[3]+fa[4])
         -5.65*Sin(-fa[2]+fa[3]+fa[4])
         -1.02*Sin(fa[2]+3*fa[4])
         +1.19*Sin(fa[4]+4*fa[5])
      +1010.16*Sin(fa[2]+fa[4])
         -1.26*Sin(fa[3]+fa[4]+2*fa[5])
        +12.12*Sin(-fa[3]+fa[4]-2*fa[5])
         -6.29*Sin(3*fa[4])
         -2.18*Sin(3*fa[4]-2*fa[5])
         +1.51*Sin(2*fa[2]+fa[4]+2*fa[5])
        +61.91*Sin(2*fa[2]+fa[4])
         +2.41*Sin(-2*fa[2]+fa[4]+4*fa[5])
         -1.62*Sin(-2*fa[2]+fa[4]+2*fa[5])
         -2.14*Sin(-2*fa[2]+fa[4]-2*fa[5])
         -7.45*Sin(fa[2]+fa[3]+fa[4]-2*fa[5])
         +5.08*Sin(-fa[2]-fa[3]+fa[4])
         +1.13*Sin(fa[2]-fa[3]+fa[4]+2*fa[5])
         -1.32*Sin(-fa[2]+fa[3]+fa[4]+2*fa[5])
         -1.77*Sin(-fa[2]+fa[3]+fa[4]-2*fa[5])
         -1.09*Sin(2*fa[3]+fa[4]-2*fa[5])
         -2.79*Sin(-fa[2]+3*fa[4])
         +3.98*Sin(3*fa[2]+fa[4])
         -1.51*Sin(3*fa[2]+fa[4]-2*fa[5])
         -1.58*Sin(-3*fa[2]+fa[4]);
  d:=6378.14/(SecRad*0.999953253*(3422.7+ar)); { for range in km }
  v:=fa[1]+SecRad*al; { longitude in radian }
  b:=SecRad*ab;       {  latitude in radian }
end;

{ approximately series for the Moon
  position in kilometer refer to mean equator of date t
  t  moment in modified julian day
  r  vector of position in refer to mean equator
  five fundamental arguments : lambda l l' F D }

procedure MoonEquPos ( t  : Extended ;  Var  r  : TVect3 ) ;
var
  al,ab,rr  : Extended ;
  ep,se,ce  : Extended ;
  xm,ym,zm  : Extended ;
  sl,cl     : Extended ;
  sb,cb     : Extended ;
begin
  ep:=ToGetEpsMean(t); { from UnForNut for mean ecliptic obliquity radian }
  se:=Sin(ep);
  ce:=Cos(ep);
  MoonEclPos(t,al,ab,rr);
  sl:=Sin(al);
  cl:=Cos(al);
  sb:=Sin(ab);
  cb:=Cos(ab);
  xm:=cb*cl;          { x unit from ecliptic to mean equator }
  ym:=cb*sl*ce-sb*se; { y unit from ecliptic to mean equator }
  zm:=cb*sl*se+sb*ce; { z unit from ecliptic to mean equator }
  r[1]:=rr*xm;        { x in km mean equator }
  r[2]:=rr*ym;        { y in km mean equator }
  r[3]:=rr*zm;        { z in km mean equator }
end;

{ position of the Moon refer to standard equator J2000.0 }

procedure MoonFixPos ( t  : Extended ;  Var  r  : TVect3 ) ;
var
  i,j  : Byte ;
  c  : TVect3 ;
  p  : TMatr33 ; { type from UnConTyp }
begin
  MoonEquPos(t,c); { the Moon position refer to mean equator }
  ClcPrecMatr(MJD2000,t,p); { procedure from UnForPnm precession matrix }
  for i:=1 to 3 do
    begin
      r[i]:=0.0;
      for j:=1 to 3 do
        r[i]:=r[i]+p[j,i]*c[j]; { refer to fixed equator }
    end;
end;

{ position of the Moon refer to true equator }

procedure MoonTruPos ( t  : Extended ;  Var  r  : TVect3 ) ;
var
  i,j  : Byte ;
  ep,dp,de  : Extended ; { obliquity nutation parameters }
  c  : TVect3 ;
  p  : TMatr33 ; { type from UnConTyp }
begin
  MoonEquPos(t,c); { the Moon position refer to mean equator }
  ep:=ToGetEpsMean(t); { function from UnForNut obliquity }
  ClcNut(t,dp,de); { procedure from UnForNut nutation parameters }
  ClcNutMatr(t,dp,de,ep,p); { procedure from UnForPnm nutation matrix }
  for i:=1 to 3 do
    begin
      r[i]:=0.0;
      for j:=1 to 3 do
        r[i]:=r[i]+p[i,j]*c[j]; { refer to true equator }
    end;
end;

{ 
  position in kilometer refer to mean equator of date t
  t  modified julian day
  v  longitude radian (lambda)  ecliptic of date
  b  latitude  radian (beta)
  r  distance  km               from UnForMup procedure MoonBrownPos
  t  moment in modified julian day
  r  vector of position in refer to mean equator
}

procedure BrmEquPos ( t , v , b , d  : Extended ;  Var  r  : TVect3 ) ;
var
  ep,se,ce  : Extended ;    { for obliquity }
  cl,sl,cb,sb  : Extended ;
begin
  ep:=ToGetEpsMean(t); { obliquity }
  se:=Sin(ep);
  ce:=Cos(ep);
  sl:=Sin(v);
  cl:=Cos(v);
  sb:=Sin(b);
  cb:=Cos(b);
  r[1]:=d*cb*cl;            { from ecliptic of date }
  r[2]:=d*(cb*sl*ce-sb*se); { to   equator  of date }
  r[3]:=d*(cb*sl*se+sb*ce);
end;

{ position of the Moon refer to standard equator J2000.0 }

procedure BrmFixPos ( t , v , b , d  : Extended ;  Var  r  : TVect3 ) ;
var
  i,j  : Byte ;
  c  : TVect3 ;
  p  : TMatr33 ; { type from UnConTyp }
begin
  BrmEquPos(t,v,b,d,c); { from the Moon ecliptic spheric position }
  ClcPrecMatr(MJD2000,t,p); { procedure from UnForPnm precession matrix }
  for i:=1 to 3 do
    begin
      r[i]:=0.0;
      for j:=1 to 3 do
        r[i]:=r[i]+p[j,i]*c[j]; { refer to fixed equator }
    end;
end;

{ position of the Moon refer to true equator }

procedure BrmTruPos ( t , v , b , d  : Extended ;  Var  r  : TVect3 ) ;
var
  i,j  : Byte ;
  ep,dp,de  : Extended ; { obliquity nutation parameters }
  c  : TVect3 ;
  p  : TMatr33 ; { type from UnConTyp }
begin
  BrmEquPos(t,v,b,d,c); { from the Moon ecliptic spheric position }
  ep:=ToGetEpsMean(t); { function from UnForNut obliquity }
  ClcNut(t,dp,de); { procedure from UnForNut nutation parameters }
  ClcNutMatr(t,dp,de,ep,p); { procedure from UnForPnm nutation matrix }
  for i:=1 to 3 do
    begin
      r[i]:=0.0;
      for j:=1 to 3 do
        r[i]:=r[i]+p[i,j]*c[j]; { refer to true equator }
    end;
end;

end.