
 UNIT UnForPNm ; { for matrix Precessin Nutation }

 INTERFACE

 Uses
   UnConTyp ; { TMatr33 }

 Procedure ClcPrecAngles ( Epoch , Tmjd  : Extended ; { angles ArcSec }
                           Var  Dzita0 , Teta , Zet  : Extended ) ;

 Procedure ClcPrecMatr ( Epoch , Tmjd  : Extended ;
                         Var  PrecMatr  : TMatr33 ) ;

 Procedure ClcNutMatr ( Tmjd : Extended ;
                        DeltaPsi  : Extended ;
                        DeltaEps  : Extended ;
                        EpsMean   : Extended ;
                        Var  MatrNut : TMatr33 );

 IMPLEMENTATION

 Uses
   UnForFun ; { procedure ForRotMatr(iaxis,angle,matr) angle radian }

 Procedure ClcPrecAngles ( Epoch , Tmjd  : Extended ; { angles ArcSec }
                           Var  Dzita0 , Teta , Zet  : Extended ) ;
 Var
   de,de2,dt,dt2,dt3,r : Extended;
 Begin
   de:=(Epoch-51544.5)/36525.0; { in century }
   de2:=Sqr(de);
   dt:=(Tmjd-Epoch)/36525.0;    { in century }
   dt2:=Sqr(dt);
   dt3:=dt2*dt;
   r:=(2306.2181+1.39656*de-0.000139*de2)*dt; { in second of arc }
   Dzita0:=r+(0.30188-0.000344*de)*dt2+0.017998*dt3;
   Teta:=(2004.3109-0.85330*de-0.000217*de2)*dt
          -(0.42665+0.000217*de)*dt2-0.041833*dt3;
   Zet:=r+(1.09468+0.000066*de)*dt2+0.018203*dt3;
 End;

  { ClcPrecMatr is a procedure
    to calculate the Matrix of precession
       for change from   Epoch  to  Tmjd  or another words
       from the fixed equator and ecliptic to the instantaneous ones.
       Fundamental  EPOCH  is  J2000.0
         but  Epoch  as parameter
              may be any fixed epoch in  MJD
                 and  Tmjd  is epoch of the Date in  MJD .
       PrecMatr[1..3,1..3] is the resulting massive .
       global const - SecRad - for change from arcsec to radians . }

 Procedure ClcPrecMatr ( Epoch , Tmjd : Extended ;
                         Var  PrecMatr : TMatr33 );
 Var
    Dzita0,Teta,Zet : Extended;
    aa,ab,ac,am  : TMatr33 ;
 Begin
    ClcPrecAngles(Epoch,Tmjd,Dzita0,Teta,Zet); { angles in ArcSec }
    ForRotMatr(3,-SecRad*Dzita0,aa); { procedure from UnForFun }
    ForRotMatr(2,SecRad*Teta,ab);    { procedure for rotation }
    ForRotMatr(3,-SecRad*Zet,ac);
    ToMultMatr(ab,aa,am); { procedure from UnForFun }
    ToMultMatr(ac,am,PrecMatr); { this is our result }
 End;

 Procedure ClcNutMatr ( Tmjd : Extended ;
                        DeltaPsi  : Extended ; { radian }
                        DeltaEps  : Extended ; { radian }
                        EpsMean   : Extended ; { radian }
                        Var  MatrNut : TMatr33 );
 Var
   i,j,k  : Byte ;
   se,ce,s,c,e  : Extended ;
   MatrEps,MatrPsi,MatrCur  : TMatr33 ; { type from UnConTyp }
 Begin
   ForRotMatr(1,EpsMean,MatrEps);   { procedure from UnForFun }
   ForRotMatr(3,-DeltaPsi,MatrPsi); { matrix of rotation }
   ToMultMatr(MatrPsi,MatrEps,MatrCur); { procedure from UnForFun }
   ForRotMatr(1,-EpsMean-DeltaEps,MatrEps);
   ToMultMatr(MatrEps,MatrCur,MatrNut); { this is our result }
 End;

 END.