
 { for partial derivation to data processing and to pseudoinverse }

 UNIT UnMatDer ;

 INTERFACE

 Uses
   UnConTyp ; { const type }

 { for pseudoinversion }
 Const
   MatrN   = 7 ; { + 3 - position }
   MatrM   = 7 ; { + 3 - velosity + 1 reflection coefficient }

 Type
   TypeDimN    = Array [1..MatrN] of Extended ;
   TypeDimM    = Array [1..MatrM] of Extended ;
   TypeDimNM   = Array [1..MatrN,1..MatrM] of Extended ;
   TypeDimMN   = Array [1..MatrM,1..MatrN] of Extended ;
   TypeDimInt  = Array [1..MatrM] of Integer ;
   TDerOneVar  = TypeDimM ;

 Type
   RecPosDer   = Record
                   TimeInMJD    : Extended ; { moment in mjd }
                   PosEtalon    : TVect3 ; { in km }
                   x , y , dut  : Extended ; { ERParam value } 
                   PosResidual  : TVect3 ; { in km }
                   PosNulloVar  : TVect3 ; { in km }
                   PosDerivats  : Array[1..3] of TDerOneVar ; { in km }
                 End;

 Const
   DifParam   : TDerOneVar  =
                    ( 1.0e-3 , 1.0e-3 , 1.0e-3 ,   { for position in km }
                      1.0e-6 , 1.0e-6 , 1.0e-6 ,   { for velosity in km/s }
                      1.0e-10 ) ; { for reflection coefficient in km/s^2 }

 Var
   DerFile    : File of RecPosDer ;
   DerRec     : RecPosDer ;
   NumPer     : LongInt ;   { count in file }
   numter     : Byte ;      { number of iterations }

 Procedure ToObtainDerivations ( tb  : Extended ;
                                 rb , vb  : TVect3 ) ;

 Procedure ClcVariablesCorrections ( Var  DifPar  : TypeDimM ) ;

 IMPLEMENTATION

 Uses
   UnForPrt , { prtf text file }
   UnForRom , { proc FromFixToTerraP with the pole position }
   UnForFun , { proc MultMatrVec }
   UnForSop , { global var SatReflCoef in km/s^2 }
   UnEveRim , { proc PosVelEverGrator }
   UnForStr , { function FloatToSingle }
   UnTreRoc , { proc VecToTerra }
   UnMatInv ; { PseudoInVerse }

 Var
   corefl : Extended ;  { refl.coef. SatReflCoef to keep value in memory }

 Procedure ToVariateOneParam ( nump  : Byte ;
                               posf , velf  : TVect3 ;
                               Var  pos , vel  : TVect3 ) ;
 Begin
   pos:=posf;
   vel:=velf;
   SatReflCoef:=corefl; { this is a value that was keep }
   Case  nump  Of
     0          :  Exit ; { no variations for nullo variant }
     1 , 2 , 3  :  pos[nump]:=posf[nump]+DifParam[nump]; { to position }
     4 , 5 , 6  :  vel[nump-3]:=velf[nump-3]+DifParam[nump]; { to velosity }
     7          :  SatReflCoef:=corefl+DifParam[nump]; { reflection coef. }
   End; { case }
 End;

 Procedure ToReWriteDerFile ( nd  : Byte ;  tc  : Extended ; rz  : TVect3 ) ;
 Var
   i  : Byte ;
   r  : TVect3 ;
 Begin
   with  DerRec  do               { procedure from UnTreRoc }
     VecToTerra(tc,x,y,dut,rz,r); { vector from fixed to terra system }
   If  nd = 0
     Then { nullo variant }
       With  DerRec  Do
         Begin
           For i:=1 To 3 Do
             Begin
               PosNulloVar[i]:=r[i];
               PosResidual[i]:=1.0e3*(PosEtalon[i]-PosNulloVar[i]); { in metr }
             End;
           if  NumPer < 5  then
             WriteLn(numter:3,nd:3,
                     tc:16:6,PosResidual[1]:16:6,
                     PosResidual[2]:16:6,PosResidual[3]:16:6,' m');
         End
     Else { with variation }
       With  DerRec  Do
         Begin
           For i:=1 To 3 Do
             PosDerivats[i][nd]:=1.0e3*(r[i]-PosNulloVar[i]); { in metr }
           if  NumPer < 5  then
             WriteLn(numter:3,nd:3,
                     tc:16:6,PosDerivats[1][nd]:16:6,
                     PosDerivats[2][nd]:16:6,PosDerivats[3][nd]:16:6,' m');
         End;
   Seek(DerFile,NumPer);
   Write(DerFile,DerRec);
 End;

 { to calculate step by step isochronic derivations 
      ds   step in second for numerical integration }

 Procedure ToObtainDerivations ( tb  : Extended ;       { initial moment }
                                 rb , vb  : TVect3 ) ;  { initial vector }
 Var
   i  : Byte ;
   dstep  : Extended ; { step to integrate }
   NumD   : Byte ;
   MaxD   : Byte ;
   tc,zt  : Extended ;
   rv,vv  : TVect3 ;
   rz,vz  : TVect3 ;
   rd,re  : TVect3 ; { type from UnConTyp }
   vd,ve  : TVect3 ; { type from UnConTyp }
   td,te  : Extended ;
 Begin
   dstep:=600.0; { step to integrate in second }
   corefl:=SatReflCoef; { global var UnForSop corefl to keep in memory }
   MaxD:=7;      { 3 position3# velosity 1 reflection coefficient }
   For NumD:=0 To MaxD Do
     Begin  { with variation for one current parametr }
       ToVariateOneParam(NumD,rb,vb,rv,vv); { vectors rb vb are fixed }
       td:=tb; { initial moment to integrate }
       rd:=rv; { initial position }
       vd:=vv; { initial velosity }
       te:=td; { moment in MJD}
       re:=rd; { position in km }
       ve:=vd; { velosity in km/s }
       EverInit ;         { from UnEveRim to start EverGrator }
       EverCoefNullo ;    { from UnEveRim to start EverGrator }
       ReSet(DerFile);
       NumPer:=-1;
       While  NOT EOF(DerFile)  Do
         Begin
           Read(DerFile,DerRec); { global variables from here }
           tc:=DerRec.TimeInMJD; { current moment for derivations }
           NumPer:=NumPer+1; { number of record to read }
           While  ( te < (tc-1.0e-8) )  Do { to integrate from UnEveRim }
             PosVelEverGrator(dstep,td,rd,vd,te,re,ve); { from UnEveRim }
           zt:=86400.0*(tc-td); { interval in second }
           ClcVectFromPolinom(zt,rd,vd,rz,vz);
           ToReWriteDerFile(NumD,tc,rz);
         End;
       Close(DerFile);
     End;
   SatReflCoef:=corefl; { global var UnForSop corefl to keep in memory }
 End;

 Procedure ToReCalcResiduals ( MMatr  : Integer ;
                               Var  DMin , DMax , DifS  : Extended ;
                               Var  DifPar  : TypeDimM ) ;
 Var
   i,j   : Byte ;
   NumP  : Integer ;
   DifA  : Extended ;
 Begin
   WriteLn(prtf);
   Write(prtf,' ':5); { simple title for table }
   Write(prtf,'        X (O-C) metr    ');
   Write(prtf,'        Y (O-C) metr    ');
   Write(prtf,'        Z (O-C) metr    ');
   WriteLn(prtf);
   Write(prtf,' ':5);
   For i:=1 To 3 Do  Write(prtf,'      before     after  ');
   For i:=1 To 2 Do  WriteLn(prtf);
   NumP:=0;
   DifS:=0;
   DMax:=-1.0e6;
   DMin:=+1.0e6;
   While  NOT EOF(DerFile)  Do
     Begin
       Read(DerFile,DerRec);
       With  DerRec  Do
         Begin
           NumP:=NumP+3;
           Write(prtf,NumP-2:5);
           For i:=1 To 3 Do
             Begin
               DifA:=PosResidual[i]; { in m }
               For j:=1 To MMatr Do
                 If  DifParam[j] > 1.0e-13  Then
                   DifA:=DifA-PosDerivats[i][j]*DifPar[j]/DifParam[j];
               Write(prtf,PosResidual[i]:12:3,DifA:10:3,' m');
               DifS:=DifS+Sqr(DifA);
               If  DifA < DMin  Then  DMin:=DifA;
               If  DifA > DMax  Then  DMax:=DifA;
             End;
         End;
       WriteLn(prtf);
     End;
 End;

 { to obtain corrections of the parameters by the least squares method }

 Procedure ClcVariablesCorrections ( Var  DifPar  : TypeDimM ) ;
 Var
   i,j,k                  : Byte ;
   NumP,
   MatrRank,NMatr,MMatr   : Integer ;
   q,DifA,DMax,DMin,
   DifS,Sigma,Epsilon     : Extended ;
   MatrVect               : TypeDimInt ;
   AMatr                  : TypeDimNM ;
   BMatr                  : TypeDimMN ;
   DifVect                : Array[1..3] of TypeDimN ;
   DParam,SigmaP          : TDerOneVar ;
 Begin
   WriteLn;
   WriteLn('matrix inverse');
   NMatr:=MatrN;
   MMatr:=MatrM;
   Epsilon:=1.0e-13;
   For i:=1 To NMatr Do
     Begin
       For k:=1 To 3 Do  DifVect[k][i]:=0.0;
       For j:=1 To MMatr Do
         Begin
           AMatr[i,j]:=0.0;
           BMatr[j,i]:=0.0;
         End;
     End;
   NumP:=0;
   ReSet(DerFile);
   NumP:=0;
   DifS:=0;
   While  NOT EOF(DerFile)  Do
     Begin
       Read(DerFile,DerRec);
       With  DerRec  Do
         Begin
           NumP:=NumP+3;
           For k:=1 To 3 Do
             DifS:=DifS+Sqr(PosResidual[k]); { in m }
           For i:=1 To NMatr Do
             Begin
               For k:=1 To 3 Do
                 DifVect[k][i]:=DifVect[k][i]
                                +PosResidual[k]*PosDerivats[k][i];
               For k:=1 To 3 Do
                 For j:=i To MMatr Do
                   AMatr[i,j]:=AMatr[i,j]
                               +PosDerivats[k][i]*PosDerivats[k][j];
             End;
         End;
     End;
   For i:=1 To NMatr Do
     For j:=i To MMatr Do
       AMatr[j,i]:=AMatr[i,j];
   WriteLn('number of points',NumP:6);
   PseudoInverse(AMatr,NMatr,MMatr,Epsilon,MatrRank,MatrVect,BMatr);
   WriteLn('rank of matrix',MatrRank:5);
   Write('on variables :');
   For i:=1 To MMatr Do  Write(MatrVect[i]:4);
   WriteLn;
   q:=NumP;
   If  q < 1.0  Then  q:=1;
   Sigma:=Sqrt(DifS/q); { in metr }
   WriteLn('  RMS error in metr',Sigma:10:3);
   WriteLn;
   For j:=1 To MMatr Do
     Begin
       q:=0;
       For i:=1 To NMatr Do
         For k:=1 To 3 Do
           q:=q+BMatr[j,i]*DifVect[k][i];
       DifPar[j]:=q*DifParam[j];
     End;
   WriteLn(' corrections to position and velosity');
   WriteLn(DifPar[1]:20:10,
           DifPar[2]:20:10,
           DifPar[3]:20:10,'  in km');
   WriteLn(DifPar[4]:20:10,
           DifPar[5]:20:10,
           DifPar[6]:20:10,'  in km/s');
   WriteLn(' correction  to reflection coefficient');
   WriteLn(FloatToSingle(DifPar[7],20):25,'  in km/s^2');     { UnForStr }
   WriteLn(prtf,' corrections to position and velosity');
   WriteLn(prtf,DifPar[1]:20:10,
                DifPar[2]:20:10,
                DifPar[3]:20:10,'  in km');
   WriteLn(prtf,DifPar[4]:20:10,
                DifPar[5]:20:10,
                DifPar[6]:20:10,'  in km/s');
   WriteLn(prtf,' correction  to reflection coefficient');
   WriteLn(prtf,FloatToSingle(DifPar[7],20):25,'  in km/s^2'); { UnForStr }
   { to recalculate residuals }
   ReSet(DerFile);
   ToReCalcResiduals(MMatr,DMin,DMax,DifS,DifPar);
   Close(DerFile);
   q:=NumP-MatrRank;
   If  q < 1.0  Then  q:=1.0;
   Sigma:=Sqrt(DifS/q);
   For j:=1 To MMatr Do
     Begin
       DParam[j]:=DifPar[j];
       SigmaP[j]:=Sigma*Sqrt(Abs(BMatr[j,j]))*DifParam[j];
     End;
   WriteLn;
   WriteLn('  RMS error in metr',Sigma:10:3);
   WriteLn('  residuals ',
           '   from',DMin:10:3,
           '     to',DMax:10:3,'  in metr');
   WriteLn(prtf);
   WriteLn(prtf,'  RMS error in metr',Sigma:10:3);
   WriteLn(prtf,'  residuals ',
           '   from',DMin:10:3,
           '     to',DMax:10:3,'  in metr');
 End;

 END.
